@@ -6,6 +6,7 @@ MYPY = TYPE_CHECKING
 
 
 SENSITIVE_DATA_SUBSTITUTE = "[Filtered]"
+BLOB_DATA_SUBSTITUTE = "[Blob substitute]"
 
 
 class AnnotatedValue:
@@ -215,13 +216,47 @@ if TYPE_CHECKING:
     # TODO: Make a proper type definition for this (PRs welcome!)
     Hint = Dict[str, Any]
 
+    AttributeValue = (
+        str
+        | bool
+        | float
+        | int
+        | list[str]
+        | list[bool]
+        | list[float]
+        | list[int]
+        | tuple[str, ...]
+        | tuple[bool, ...]
+        | tuple[float, ...]
+        | tuple[int, ...]
+    )
+    Attributes = dict[str, AttributeValue]
+
+    SerializedAttributeValue = TypedDict(
+        # https://develop.sentry.dev/sdk/telemetry/attributes/#supported-types
+        "SerializedAttributeValue",
+        {
+            "type": Literal[
+                "string",
+                "boolean",
+                "double",
+                "integer",
+                "string[]",
+                "boolean[]",
+                "double[]",
+                "integer[]",
+            ],
+            "value": AttributeValue,
+        },
+    )
+
     Log = TypedDict(
         "Log",
         {
             "severity_text": str,
             "severity_number": int,
             "body": str,
-            "attributes": dict[str, str | bool | float | int],
+            "attributes": Attributes,
             "time_unix_nano": int,
             "trace_id": Optional[str],
             "span_id": Optional[str],
@@ -229,14 +264,7 @@ if TYPE_CHECKING:
     )
 
     MetricType = Literal["counter", "gauge", "distribution"]
-
-    MetricAttributeValue = TypedDict(
-        "MetricAttributeValue",
-        {
-            "value": Union[str, bool, float, int],
-            "type": Literal["string", "boolean", "double", "integer"],
-        },
-    )
+    MetricUnit = Union[DurationUnit, InformationUnit, str]
 
     Metric = TypedDict(
         "Metric",
@@ -247,8 +275,8 @@ if TYPE_CHECKING:
             "name": str,
             "type": MetricType,
             "value": float,
-            "unit": Optional[str],
-            "attributes": dict[str, str | bool | float | int],
+            "unit": Optional[MetricUnit],
+            "attributes": Attributes,
         },
     )
 
