@@ -81,6 +81,7 @@ _AUTO_ENABLING_INTEGRATIONS = [
     "sentry_sdk.integrations.fastapi.FastApiIntegration",
     "sentry_sdk.integrations.flask.FlaskIntegration",
     "sentry_sdk.integrations.gql.GQLIntegration",
+    "sentry_sdk.integrations.google_genai.GoogleGenAIIntegration",
     "sentry_sdk.integrations.graphene.GrapheneIntegration",
     "sentry_sdk.integrations.httpx.HttpxIntegration",
     "sentry_sdk.integrations.huey.HueyIntegration",
@@ -148,6 +149,7 @@ _MIN_VERSIONS = {
     "openai_agents": (0, 0, 19),
     "openfeature": (0, 7, 1),
     "pydantic_ai": (1, 0, 0),
+    "pymongo": (3, 5, 0),
     "quart": (0, 16, 0),
     "ray": (2, 7, 0),
     "requests": (2, 0, 0),
