@@ -1,13 +1,17 @@
+import sys
+from collections.abc import Iterable
 from functools import wraps
 from typing import TYPE_CHECKING
 
 import sentry_sdk
 from sentry_sdk.ai.monitoring import record_token_usage
 from sentry_sdk.ai.utils import (
+    GEN_AI_ALLOWED_MESSAGE_ROLES,
     set_data_normalized,
     normalize_message_roles,
     truncate_and_annotate_messages,
     get_start_span_function,
+    transform_anthropic_content_part,
 )
 from sentry_sdk.consts import OP, SPANDATA, SPANSTATUS
 from sentry_sdk.integrations import _check_minimum_version, DidNotEnable, Integration
@@ -18,6 +22,7 @@ from sentry_sdk.utils import (
     event_from_exception,
     package_version,
     safe_serialize,
+    reraise,
 )
 
 try:
@@ -39,7 +44,7 @@ except ImportError:
     raise DidNotEnable("Anthropic not installed")
 
 if TYPE_CHECKING:
-    from typing import Any, AsyncIterator, Iterator
+    from typing import Any, AsyncIterator, Iterator, List, Optional, Union
     from sentry_sdk.tracing import Span
 
 
@@ -47,13 +52,11 @@ class AnthropicIntegration(Integration):
     identifier = "anthropic"
     origin = f"auto.ai.{identifier}"
 
-    def __init__(self, include_prompts=True):
-        # type: (AnthropicIntegration, bool) -> None
+    def __init__(self: "AnthropicIntegration", include_prompts: bool = True) -> None:
         self.include_prompts = include_prompts
 
     @staticmethod
-    def setup_once():
-        # type: () -> None
+    def setup_once() -> None:
         version = package_version("anthropic")
         _check_minimum_version(AnthropicIntegration, version)
 
@@ -61,8 +64,7 @@ class AnthropicIntegration(Integration):
         AsyncMessages.create = _wrap_message_create_async(AsyncMessages.create)
 
 
-def _capture_exception(exc):
-    # type: (Any) -> None
+def _capture_exception(exc: "Any") -> None:
     set_span_errored()
 
     event, hint = event_from_exception(
@@ -73,25 +75,47 @@ def _capture_exception(exc):
     sentry_sdk.capture_event(event, hint=hint)
 
 
-def _get_token_usage(result):
-    # type: (Messages) -> tuple[int, int]
+def _get_token_usage(result: "Messages") -> "tuple[int, int, int, int]":
     """
     Get token usage from the Anthropic response.
+    Returns: (input_tokens, output_tokens, cache_read_input_tokens, cache_write_input_tokens)
     """
     input_tokens = 0
     output_tokens = 0
+    cache_read_input_tokens = 0
+    cache_write_input_tokens = 0
     if hasattr(result, "usage"):
         usage = result.usage
         if hasattr(usage, "input_tokens") and isinstance(usage.input_tokens, int):
             input_tokens = usage.input_tokens
         if hasattr(usage, "output_tokens") and isinstance(usage.output_tokens, int):
             output_tokens = usage.output_tokens
+        if hasattr(usage, "cache_read_input_tokens") and isinstance(
+            usage.cache_read_input_tokens, int
+        ):
+            cache_read_input_tokens = usage.cache_read_input_tokens
+        if hasattr(usage, "cache_creation_input_tokens") and isinstance(
+            usage.cache_creation_input_tokens, int
+        ):
+            cache_write_input_tokens = usage.cache_creation_input_tokens
 
-    return input_tokens, output_tokens
+    return (
+        input_tokens,
+        output_tokens,
+        cache_read_input_tokens,
+        cache_write_input_tokens,
+    )
 
 
-def _collect_ai_data(event, model, input_tokens, output_tokens, content_blocks):
-    # type: (MessageStreamEvent, str | None, int, int, list[str]) -> tuple[str | None, int, int, list[str]]
+def _collect_ai_data(
+    event: "MessageStreamEvent",
+    model: "str | None",
+    input_tokens: int,
+    output_tokens: int,
+    cache_read_input_tokens: int,
+    cache_write_input_tokens: int,
+    content_blocks: "list[str]",
+) -> "tuple[str | None, int, int, int, int, list[str]]":
     """
     Collect model information, token usage, and collect content blocks from the AI streaming response.
     """
@@ -101,6 +125,14 @@ def _collect_ai_data(event, model, input_tokens, output_tokens, content_blocks):
                 usage = event.message.usage
                 input_tokens += usage.input_tokens
                 output_tokens += usage.output_tokens
+                if hasattr(usage, "cache_read_input_tokens") and isinstance(
+                    usage.cache_read_input_tokens, int
+                ):
+                    cache_read_input_tokens += usage.cache_read_input_tokens
+                if hasattr(usage, "cache_creation_input_tokens") and isinstance(
+                    usage.cache_creation_input_tokens, int
+                ):
+                    cache_write_input_tokens += usage.cache_creation_input_tokens
                 model = event.message.model or model
             elif event.type == "content_block_start":
                 pass
@@ -114,14 +146,45 @@ def _collect_ai_data(event, model, input_tokens, output_tokens, content_blocks):
             elif event.type == "message_delta":
                 output_tokens += event.usage.output_tokens
 
-    return model, input_tokens, output_tokens, content_blocks
+    return (
+        model,
+        input_tokens,
+        output_tokens,
+        cache_read_input_tokens,
+        cache_write_input_tokens,
+        content_blocks,
+    )
 
 
-def _set_input_data(span, kwargs, integration):
-    # type: (Span, dict[str, Any], AnthropicIntegration) -> None
+def _transform_anthropic_content_block(
+    content_block: "dict[str, Any]",
+) -> "dict[str, Any]":
+    """
+    Transform an Anthropic content block using the Anthropic-specific transformer,
+    with special handling for Anthropic's text-type documents.
+    """
+    # Handle Anthropic's text-type documents specially (not covered by shared function)
+    if content_block.get("type") == "document":
+        source = content_block.get("source")
+        if isinstance(source, dict) and source.get("type") == "text":
+            return {
+                "type": "text",
+                "text": source.get("data", ""),
+            }
+
+    # Use Anthropic-specific transformation
+    result = transform_anthropic_content_part(content_block)
+    return result if result is not None else content_block
+
+
+def _set_input_data(
+    span: "Span", kwargs: "dict[str, Any]", integration: "AnthropicIntegration"
+) -> None:
     """
     Set input data for the span based on the provided keyword arguments for the anthropic message creation.
     """
+    set_data_normalized(span, SPANDATA.GEN_AI_OPERATION_NAME, "chat")
+    system_prompt = kwargs.get("system")
     messages = kwargs.get("messages")
     if (
         messages is not None
@@ -130,25 +193,69 @@ def _set_input_data(span, kwargs, integration):
         and integration.include_prompts
     ):
         normalized_messages = []
+        if system_prompt:
+            system_prompt_content: "Optional[Union[str, List[dict[str, Any]]]]" = None
+            if isinstance(system_prompt, str):
+                system_prompt_content = system_prompt
+            elif isinstance(system_prompt, Iterable):
+                system_prompt_content = []
+                for item in system_prompt:
+                    if (
+                        isinstance(item, dict)
+                        and item.get("type") == "text"
+                        and item.get("text")
+                    ):
+                        system_prompt_content.append(item.copy())
+
+            if system_prompt_content:
+                normalized_messages.append(
+                    {
+                        "role": GEN_AI_ALLOWED_MESSAGE_ROLES.SYSTEM,
+                        "content": system_prompt_content,
+                    }
+                )
+
         for message in messages:
             if (
-                message.get("role") == "user"
+                message.get("role") == GEN_AI_ALLOWED_MESSAGE_ROLES.USER
                 and "content" in message
                 and isinstance(message["content"], (list, tuple))
             ):
+                transformed_content = []
                 for item in message["content"]:
-                    if item.get("type") == "tool_result":
-                        normalized_messages.append(
-                            {
-                                "role": "tool",
-                                "content": {
-                                    "tool_use_id": item.get("tool_use_id"),
-                                    "output": item.get("content"),
-                                },
-                            }
-                        )
+                    # Skip tool_result items - they can contain images/documents
+                    # with nested structures that are difficult to redact properly
+                    if isinstance(item, dict) and item.get("type") == "tool_result":
+                        continue
+
+                    # Transform content blocks (images, documents, etc.)
+                    transformed_content.append(
+                        _transform_anthropic_content_block(item)
+                        if isinstance(item, dict)
+                        else item
+                    )
+
+                # If there are non-tool-result items, add them as a message
+                if transformed_content:
+                    normalized_messages.append(
+                        {
+                            "role": message.get("role"),
+                            "content": transformed_content,
+                        }
+                    )
             else:
-                normalized_messages.append(message)
+                # Transform content for non-list messages or assistant messages
+                transformed_message = message.copy()
+                if "content" in transformed_message:
+                    content = transformed_message["content"]
+                    if isinstance(content, (list, tuple)):
+                        transformed_message["content"] = [
+                            _transform_anthropic_content_block(item)
+                            if isinstance(item, dict)
+                            else item
+                            for item in content
+                        ]
+                normalized_messages.append(transformed_message)
 
         role_normalized_messages = normalize_message_roles(normalized_messages)
         scope = sentry_sdk.get_current_scope()
@@ -186,23 +293,24 @@ def _set_input_data(span, kwargs, integration):
 
 
 def _set_output_data(
-    span,
-    integration,
-    model,
-    input_tokens,
-    output_tokens,
-    content_blocks,
-    finish_span=False,
-):
-    # type: (Span, AnthropicIntegration, str | None, int | None, int | None, list[Any], bool) -> None
+    span: "Span",
+    integration: "AnthropicIntegration",
+    model: "str | None",
+    input_tokens: "int | None",
+    output_tokens: "int | None",
+    cache_read_input_tokens: "int | None",
+    cache_write_input_tokens: "int | None",
+    content_blocks: "list[Any]",
+    finish_span: bool = False,
+) -> None:
     """
     Set output data for the span based on the AI response."""
     span.set_data(SPANDATA.GEN_AI_RESPONSE_MODEL, model)
     if should_send_default_pii() and integration.include_prompts:
-        output_messages = {
+        output_messages: "dict[str, list[Any]]" = {
             "response": [],
             "tool": [],
-        }  # type: (dict[str, list[Any]])
+        }
 
         for output in content_blocks:
             if output["type"] == "text":
@@ -227,14 +335,15 @@ def _set_output_data(
         span,
         input_tokens=input_tokens,
         output_tokens=output_tokens,
+        input_tokens_cached=cache_read_input_tokens,
+        input_tokens_cache_write=cache_write_input_tokens,
     )
 
     if finish_span:
         span.__exit__(None, None, None)
 
 
-def _sentry_patched_create_common(f, *args, **kwargs):
-    # type: (Any, *Any, **Any) -> Any
+def _sentry_patched_create_common(f: "Any", *args: "Any", **kwargs: "Any") -> "Any":
     integration = kwargs.pop("integration")
     if integration is None:
         return f(*args, **kwargs)
@@ -262,7 +371,12 @@ def _sentry_patched_create_common(f, *args, **kwargs):
 
     with capture_internal_exceptions():
         if hasattr(result, "content"):
-            input_tokens, output_tokens = _get_token_usage(result)
+            (
+                input_tokens,
+                output_tokens,
+                cache_read_input_tokens,
+                cache_write_input_tokens,
+            ) = _get_token_usage(result)
 
             content_blocks = []
             for content_block in result.content:
@@ -279,6 +393,8 @@ def _sentry_patched_create_common(f, *args, **kwargs):
                 model=getattr(result, "model", None),
                 input_tokens=input_tokens,
                 output_tokens=output_tokens,
+                cache_read_input_tokens=cache_read_input_tokens,
+                cache_write_input_tokens=cache_write_input_tokens,
                 content_blocks=content_blocks,
                 finish_span=True,
             )
@@ -287,18 +403,30 @@ def _sentry_patched_create_common(f, *args, **kwargs):
         elif hasattr(result, "_iterator"):
             old_iterator = result._iterator
 
-            def new_iterator():
-                # type: () -> Iterator[MessageStreamEvent]
+            def new_iterator() -> "Iterator[MessageStreamEvent]":
                 model = None
                 input_tokens = 0
                 output_tokens = 0
-                content_blocks = []  # type: list[str]
+                cache_read_input_tokens = 0
+                cache_write_input_tokens = 0
+                content_blocks: "list[str]" = []
 
                 for event in old_iterator:
-                    model, input_tokens, output_tokens, content_blocks = (
-                        _collect_ai_data(
-                            event, model, input_tokens, output_tokens, content_blocks
-                        )
+                    (
+                        model,
+                        input_tokens,
+                        output_tokens,
+                        cache_read_input_tokens,
+                        cache_write_input_tokens,
+                        content_blocks,
+                    ) = _collect_ai_data(
+                        event,
+                        model,
+                        input_tokens,
+                        output_tokens,
+                        cache_read_input_tokens,
+                        cache_write_input_tokens,
+                        content_blocks,
                     )
                     yield event
 
@@ -308,22 +436,36 @@ def _sentry_patched_create_common(f, *args, **kwargs):
                     model=model,
                     input_tokens=input_tokens,
                     output_tokens=output_tokens,
+                    cache_read_input_tokens=cache_read_input_tokens,
+                    cache_write_input_tokens=cache_write_input_tokens,
                     content_blocks=[{"text": "".join(content_blocks), "type": "text"}],
                     finish_span=True,
                 )
 
-            async def new_iterator_async():
-                # type: () -> AsyncIterator[MessageStreamEvent]
+            async def new_iterator_async() -> "AsyncIterator[MessageStreamEvent]":
                 model = None
                 input_tokens = 0
                 output_tokens = 0
-                content_blocks = []  # type: list[str]
+                cache_read_input_tokens = 0
+                cache_write_input_tokens = 0
+                content_blocks: "list[str]" = []
 
                 async for event in old_iterator:
-                    model, input_tokens, output_tokens, content_blocks = (
-                        _collect_ai_data(
-                            event, model, input_tokens, output_tokens, content_blocks
-                        )
+                    (
+                        model,
+                        input_tokens,
+                        output_tokens,
+                        cache_read_input_tokens,
+                        cache_write_input_tokens,
+                        content_blocks,
+                    ) = _collect_ai_data(
+                        event,
+                        model,
+                        input_tokens,
+                        output_tokens,
+                        cache_read_input_tokens,
+                        cache_write_input_tokens,
+                        content_blocks,
                     )
                     yield event
 
@@ -333,6 +475,8 @@ def _sentry_patched_create_common(f, *args, **kwargs):
                     model=model,
                     input_tokens=input_tokens,
                     output_tokens=output_tokens,
+                    cache_read_input_tokens=cache_read_input_tokens,
+                    cache_write_input_tokens=cache_write_input_tokens,
                     content_blocks=[{"text": "".join(content_blocks), "type": "text"}],
                     finish_span=True,
                 )
@@ -349,10 +493,8 @@ def _sentry_patched_create_common(f, *args, **kwargs):
     return result
 
 
-def _wrap_message_create(f):
-    # type: (Any) -> Any
-    def _execute_sync(f, *args, **kwargs):
-        # type: (Any, *Any, **Any) -> Any
+def _wrap_message_create(f: "Any") -> "Any":
+    def _execute_sync(f: "Any", *args: "Any", **kwargs: "Any") -> "Any":
         gen = _sentry_patched_create_common(f, *args, **kwargs)
 
         try:
@@ -364,16 +506,17 @@ def _wrap_message_create(f):
             try:
                 result = f(*args, **kwargs)
             except Exception as exc:
-                _capture_exception(exc)
-                raise exc from None
+                exc_info = sys.exc_info()
+                with capture_internal_exceptions():
+                    _capture_exception(exc)
+                reraise(*exc_info)
 
             return gen.send(result)
         except StopIteration as e:
             return e.value
 
     @wraps(f)
-    def _sentry_patched_create_sync(*args, **kwargs):
-        # type: (*Any, **Any) -> Any
+    def _sentry_patched_create_sync(*args: "Any", **kwargs: "Any") -> "Any":
         integration = sentry_sdk.get_client().get_integration(AnthropicIntegration)
         kwargs["integration"] = integration
 
@@ -381,17 +524,15 @@ def _wrap_message_create(f):
             return _execute_sync(f, *args, **kwargs)
         finally:
             span = sentry_sdk.get_current_span()
-            if span is not None and span.status == SPANSTATUS.ERROR:
+            if span is not None and span.status == SPANSTATUS.INTERNAL_ERROR:
                 with capture_internal_exceptions():
                     span.__exit__(None, None, None)
 
     return _sentry_patched_create_sync
 
 
-def _wrap_message_create_async(f):
-    # type: (Any) -> Any
-    async def _execute_async(f, *args, **kwargs):
-        # type: (Any, *Any, **Any) -> Any
+def _wrap_message_create_async(f: "Any") -> "Any":
+    async def _execute_async(f: "Any", *args: "Any", **kwargs: "Any") -> "Any":
         gen = _sentry_patched_create_common(f, *args, **kwargs)
 
         try:
@@ -403,16 +544,17 @@ def _wrap_message_create_async(f):
             try:
                 result = await f(*args, **kwargs)
             except Exception as exc:
-                _capture_exception(exc)
-                raise exc from None
+                exc_info = sys.exc_info()
+                with capture_internal_exceptions():
+                    _capture_exception(exc)
+                reraise(*exc_info)
 
             return gen.send(result)
         except StopIteration as e:
             return e.value
 
     @wraps(f)
-    async def _sentry_patched_create_async(*args, **kwargs):
-        # type: (*Any, **Any) -> Any
+    async def _sentry_patched_create_async(*args: "Any", **kwargs: "Any") -> "Any":
         integration = sentry_sdk.get_client().get_integration(AnthropicIntegration)
         kwargs["integration"] = integration
 
@@ -420,15 +562,14 @@ def _wrap_message_create_async(f):
             return await _execute_async(f, *args, **kwargs)
         finally:
             span = sentry_sdk.get_current_span()
-            if span is not None and span.status == SPANSTATUS.ERROR:
+            if span is not None and span.status == SPANSTATUS.INTERNAL_ERROR:
                 with capture_internal_exceptions():
                     span.__exit__(None, None, None)
 
     return _sentry_patched_create_async
 
 
-def _is_given(obj):
-    # type: (Any) -> bool
+def _is_given(obj: "Any") -> bool:
     """
     Check for givenness safely across different anthropic versions.
     """
