@@ -1,3 +1,5 @@
+import functools
+
 import sentry_sdk
 from sentry_sdk.tracing import SOURCE_FOR_STYLE
 from sentry_sdk.utils import (
@@ -7,13 +9,20 @@ from sentry_sdk.utils import (
     parse_version,
     transaction_from_function,
 )
-from sentry_sdk.integrations import Integration, DidNotEnable
+from sentry_sdk.integrations import (
+    Integration,
+    DidNotEnable,
+    _DEFAULT_FAILED_REQUEST_STATUS_CODES,
+    _check_minimum_version,
+)
 from sentry_sdk.integrations.wsgi import SentryWsgiMiddleware
 from sentry_sdk.integrations._wsgi_common import RequestExtractor
-from sentry_sdk.scope import Scope
-from sentry_sdk._types import TYPE_CHECKING
+
+from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
+    from collections.abc import Set
+
     from sentry_sdk.integrations.wsgi import _ScopedResponse
     from typing import Any
     from typing import Dict
@@ -26,9 +35,9 @@ if TYPE_CHECKING:
 try:
     from bottle import (
         Bottle,
+        HTTPResponse,
         Route,
         request as bottle_request,
-        HTTPResponse,
         __version__ as BOTTLE_VERSION,
     )
 except ImportError:
@@ -40,49 +49,53 @@ TRANSACTION_STYLE_VALUES = ("endpoint", "url")
 
 class BottleIntegration(Integration):
     identifier = "bottle"
+    origin = f"auto.http.{identifier}"
 
     transaction_style = ""
 
-    def __init__(self, transaction_style="endpoint"):
-        # type: (str) -> None
-
+    def __init__(
+        self,
+        transaction_style: str = "endpoint",
+        *,
+        failed_request_status_codes: "Set[int]" = _DEFAULT_FAILED_REQUEST_STATUS_CODES,
+    ) -> None:
         if transaction_style not in TRANSACTION_STYLE_VALUES:
             raise ValueError(
                 "Invalid value for transaction_style: %s (must be in %s)"
                 % (transaction_style, TRANSACTION_STYLE_VALUES)
             )
         self.transaction_style = transaction_style
+        self.failed_request_status_codes = failed_request_status_codes
 
     @staticmethod
-    def setup_once():
-        # type: () -> None
+    def setup_once() -> None:
         version = parse_version(BOTTLE_VERSION)
-
-        if version is None:
-            raise DidNotEnable("Unparsable Bottle version: {}".format(BOTTLE_VERSION))
-
-        if version < (0, 12):
-            raise DidNotEnable("Bottle 0.12 or newer required.")
+        _check_minimum_version(BottleIntegration, version)
 
         old_app = Bottle.__call__
 
         @ensure_integration_enabled(BottleIntegration, old_app)
-        def sentry_patched_wsgi_app(self, environ, start_response):
-            # type: (Any, Dict[str, str], Callable[..., Any]) -> _ScopedResponse
-            return SentryWsgiMiddleware(lambda *a, **kw: old_app(self, *a, **kw))(
-                environ, start_response
+        def sentry_patched_wsgi_app(
+            self: "Any", environ: "Dict[str, str]", start_response: "Callable[..., Any]"
+        ) -> "_ScopedResponse":
+            middleware = SentryWsgiMiddleware(
+                lambda *a, **kw: old_app(self, *a, **kw),
+                span_origin=BottleIntegration.origin,
             )
 
+            return middleware(environ, start_response)
+
         Bottle.__call__ = sentry_patched_wsgi_app
 
         old_handle = Bottle._handle
 
-        @ensure_integration_enabled(BottleIntegration, old_handle)
-        def _patched_handle(self, environ):
-            # type: (Bottle, Dict[str, Any]) -> Any
+        @functools.wraps(old_handle)
+        def _patched_handle(self: "Bottle", environ: "Dict[str, Any]") -> "Any":
             integration = sentry_sdk.get_client().get_integration(BottleIntegration)
+            if integration is None:
+                return old_handle(self, environ)
 
-            scope = Scope.get_isolation_scope()
+            scope = sentry_sdk.get_isolation_scope()
             scope._name = "bottle"
             scope.add_event_processor(
                 _make_request_event_processor(self, bottle_request, integration)
@@ -95,28 +108,29 @@ class BottleIntegration(Integration):
 
         old_make_callback = Route._make_callback
 
-        @ensure_integration_enabled(BottleIntegration, old_make_callback)
-        def patched_make_callback(self, *args, **kwargs):
-            # type: (Route, *object, **object) -> Any
-            client = sentry_sdk.get_client()
+        @functools.wraps(old_make_callback)
+        def patched_make_callback(
+            self: "Route", *args: object, **kwargs: object
+        ) -> "Any":
             prepared_callback = old_make_callback(self, *args, **kwargs)
 
-            def wrapped_callback(*args, **kwargs):
-                # type: (*object, **object) -> Any
+            integration = sentry_sdk.get_client().get_integration(BottleIntegration)
+            if integration is None:
+                return prepared_callback
 
+            def wrapped_callback(*args: object, **kwargs: object) -> "Any":
                 try:
                     res = prepared_callback(*args, **kwargs)
-                except HTTPResponse:
-                    raise
                 except Exception as exception:
-                    event, hint = event_from_exception(
-                        exception,
-                        client_options=client.options,
-                        mechanism={"type": "bottle", "handled": False},
-                    )
-                    sentry_sdk.capture_event(event, hint=hint)
+                    _capture_exception(exception, handled=False)
                     raise exception
 
+                if (
+                    isinstance(res, HTTPResponse)
+                    and res.status_code in integration.failed_request_status_codes
+                ):
+                    _capture_exception(res, handled=True)
+
                 return res
 
             return wrapped_callback
@@ -125,59 +139,59 @@ class BottleIntegration(Integration):
 
 
 class BottleRequestExtractor(RequestExtractor):
-    def env(self):
-        # type: () -> Dict[str, str]
+    def env(self) -> "Dict[str, str]":
         return self.request.environ
 
-    def cookies(self):
-        # type: () -> Dict[str, str]
+    def cookies(self) -> "Dict[str, str]":
         return self.request.cookies
 
-    def raw_data(self):
-        # type: () -> bytes
+    def raw_data(self) -> bytes:
         return self.request.body.read()
 
-    def form(self):
-        # type: () -> FormsDict
+    def form(self) -> "FormsDict":
         if self.is_json():
             return None
         return self.request.forms.decode()
 
-    def files(self):
-        # type: () -> Optional[Dict[str, str]]
+    def files(self) -> "Optional[Dict[str, str]]":
         if self.is_json():
             return None
 
         return self.request.files
 
-    def size_of_file(self, file):
-        # type: (FileUpload) -> int
+    def size_of_file(self, file: "FileUpload") -> int:
         return file.content_length
 
 
-def _set_transaction_name_and_source(event, transaction_style, request):
-    # type: (Event, str, Any) -> None
+def _set_transaction_name_and_source(
+    event: "Event", transaction_style: str, request: "Any"
+) -> None:
     name = ""
 
     if transaction_style == "url":
-        name = request.route.rule or ""
+        try:
+            name = request.route.rule or ""
+        except RuntimeError:
+            pass
 
     elif transaction_style == "endpoint":
-        name = (
-            request.route.name
-            or transaction_from_function(request.route.callback)
-            or ""
-        )
+        try:
+            name = (
+                request.route.name
+                or transaction_from_function(request.route.callback)
+                or ""
+            )
+        except RuntimeError:
+            pass
 
     event["transaction"] = name
     event["transaction_info"] = {"source": SOURCE_FOR_STYLE[transaction_style]}
 
 
-def _make_request_event_processor(app, request, integration):
-    # type: (Bottle, LocalRequest, BottleIntegration) -> EventProcessor
-
-    def event_processor(event, hint):
-        # type: (Event, dict[str, Any]) -> Event
+def _make_request_event_processor(
+    app: "Bottle", request: "LocalRequest", integration: "BottleIntegration"
+) -> "EventProcessor":
+    def event_processor(event: "Event", hint: "dict[str, Any]") -> "Event":
         _set_transaction_name_and_source(event, integration.transaction_style, request)
 
         with capture_internal_exceptions():
@@ -186,3 +200,12 @@ def _make_request_event_processor(app, request, integration):
         return event
 
     return event_processor
+
+
+def _capture_exception(exception: BaseException, handled: bool) -> None:
+    event, hint = event_from_exception(
+        exception,
+        client_options=sentry_sdk.get_client().options,
+        mechanism={"type": "bottle", "handled": handled},
+    )
+    sentry_sdk.capture_event(event, hint=hint)
