@@ -1,3 +1,4 @@
+import sys
 from functools import wraps
 
 import sentry_sdk
@@ -16,6 +17,7 @@ from sentry_sdk.utils import (
     capture_internal_exceptions,
     event_from_exception,
     safe_serialize,
+    reraise,
 )
 
 from typing import TYPE_CHECKING
@@ -84,9 +86,12 @@ class OpenAIIntegration(Integration):
             AsyncResponses.create = _wrap_async_responses_create(AsyncResponses.create)
 
     def count_tokens(self: "OpenAIIntegration", s: str) -> int:
-        if self.tiktoken_encoding is not None:
+        if self.tiktoken_encoding is None:
+            return 0
+        try:
             return len(self.tiktoken_encoding.encode_ordinary(s))
-        return 0
+        except Exception:
+            return 0
 
 
 def _capture_exception(exc: "Any", manual_span_cleanup: bool = True) -> None:
@@ -157,8 +162,8 @@ def _calculate_token_usage(
                 output_tokens += count_tokens(message)
         elif hasattr(response, "choices"):
             for choice in response.choices:
-                if hasattr(choice, "message"):
-                    output_tokens += count_tokens(choice.message)
+                if hasattr(choice, "message") and hasattr(choice.message, "content"):
+                    output_tokens += count_tokens(choice.message.content)
 
     # Do not set token data if it is 0
     input_tokens = input_tokens or None
@@ -480,8 +485,10 @@ def _wrap_chat_completion_create(f: "Callable[..., Any]") -> "Callable[..., Any]
             try:
                 result = f(*args, **kwargs)
             except Exception as e:
-                _capture_exception(e)
-                raise e from None
+                exc_info = sys.exc_info()
+                with capture_internal_exceptions():
+                    _capture_exception(e)
+                reraise(*exc_info)
 
             return gen.send(result)
         except StopIteration as e:
@@ -512,8 +519,10 @@ def _wrap_async_chat_completion_create(f: "Callable[..., Any]") -> "Callable[...
             try:
                 result = await f(*args, **kwargs)
             except Exception as e:
-                _capture_exception(e)
-                raise e from None
+                exc_info = sys.exc_info()
+                with capture_internal_exceptions():
+                    _capture_exception(e)
+                reraise(*exc_info)
 
             return gen.send(result)
         except StopIteration as e:
@@ -566,8 +575,10 @@ def _wrap_embeddings_create(f: "Any") -> "Any":
             try:
                 result = f(*args, **kwargs)
             except Exception as e:
-                _capture_exception(e, manual_span_cleanup=False)
-                raise e from None
+                exc_info = sys.exc_info()
+                with capture_internal_exceptions():
+                    _capture_exception(e, manual_span_cleanup=False)
+                reraise(*exc_info)
 
             return gen.send(result)
         except StopIteration as e:
@@ -597,8 +608,10 @@ def _wrap_async_embeddings_create(f: "Any") -> "Any":
             try:
                 result = await f(*args, **kwargs)
             except Exception as e:
-                _capture_exception(e, manual_span_cleanup=False)
-                raise e from None
+                exc_info = sys.exc_info()
+                with capture_internal_exceptions():
+                    _capture_exception(e, manual_span_cleanup=False)
+                reraise(*exc_info)
 
             return gen.send(result)
         except StopIteration as e:
@@ -652,8 +665,10 @@ def _wrap_responses_create(f: "Any") -> "Any":
             try:
                 result = f(*args, **kwargs)
             except Exception as e:
-                _capture_exception(e)
-                raise e from None
+                exc_info = sys.exc_info()
+                with capture_internal_exceptions():
+                    _capture_exception(e)
+                reraise(*exc_info)
 
             return gen.send(result)
         except StopIteration as e:
@@ -683,8 +698,10 @@ def _wrap_async_responses_create(f: "Any") -> "Any":
             try:
                 result = await f(*args, **kwargs)
             except Exception as e:
-                _capture_exception(e)
-                raise e from None
+                exc_info = sys.exc_info()
+                with capture_internal_exceptions():
+                    _capture_exception(e)
+                reraise(*exc_info)
 
             return gen.send(result)
         except StopIteration as e:
